#pragma once

#define NEX5_VARIABLE_TYPE_NEURON (0)
#define NEX5_VARIABLE_TYPE_EVENT (1)
#define NEX5_VARIABLE_TYPE_INTERVAL (2)
#define NEX5_VARIABLE_TYPE_WAVEFORM (3)
#define NEX5_VARIABLE_TYPE_POPULATION_VECTOR (4)
#define NEX5_VARIABLE_TYPE_CONTINUOUS (5)
#define NEX5_VARIABLE_TYPE_MARKER (6)

#pragma pack(push, 1)

// .nex5 format is designed to be similar to .nex format so that the code that reads and writes .nex files
// would need only small modifications to implement reading and writing .nex5 files.
// .nex5 headers contain minimum information needed to store, load and properly interpret data.
// all the metadata that is rarely used in analysis 
// (for example, recording start time, recording software, electrode location, etc.)
// is stored in a string at the end of the file after all the binary data.
// metadata is stored in JSON format. see Nex5Metadata.txt for metadata description.

// .nex5 file header structure
struct Nex5FileHeader 
{
    int Nex5MagicNumber; // string NEX5; numeric value: 894977358 decimal or 0x3558454e hex

    int Nex5FileVersion;  // valid values are 500, 501 or 052 (April, 2017)
    // if 500, RecordingEndTimeInTicks is not specified
    // if 501 or greater, RecordingEndTimeInTicks is specified
    // if 502 or greater, timestamps are saved as 64-bit integers

    char Comment[256]; // file comment; UTF-8 encoding is assumed.

    double TimestampFrequency;  // timestamps frequency, in Hertz
    // timestamp values are stored in ticks, where tick = 1/Frequency.

    long long RecordingStartTimeInTicks;	// when data is imported from a file with non-zero start time 

    int NumberOfVariables;    // number of variables in the file, number of variables headers that follow this file header

    unsigned long long MetadataOffset; // where the optional metadata for the file and its variables is located in the file
    // metadata is a string in JSON format. see metadata description below. 

    long long RecordingEndTimeInTicks;	// the maximum data timestamp across all file variables (file version 501 or greater)
    // for neuron, event or marker variable, it is the last timestamp
    // for interval variable, it is the timestamp of the end of the last interval
    // for continuous variable, it is the timestamp of the last data point
    // for waveform variable, it is the timestamp of the last data point of the last waveform

    char Padding[56]; // padding for future expansion 
};

// .nex5 file variable header structure
struct Nex5VarHeader 
{
    int Type; // 0 - neuron, 1 - event, 2- interval, 3 - waveform, 4 - pop. vector, 5 - continuously recorded, 6 - marker
    // see #define statements above

    int Version; // Version = 500 as of July 2015 (version 5.016).

    char Name[64]; // variable name. it is assumed that variable names are unique within the file. 
    // that is, there are no two variables with the same name.
    // there is one reserved name: AllFile 
    // NeuroExplorer creates AllFile variable from file data. 
    // It is an interval variable with a single interval [RecordingStartTimeInTicks, RecordingEndTimeInTicks]

    unsigned long long DataOffset; // where the data array for this variable is located in the file

    unsigned long long Count; // neuron variable: number of timestamps
    // event variable: number of timestamps
    // interval variable: number of intervals
    // waveform variable: number of waveforms
    // continuous variable: number of fragments
    // population vector: number of weights

    int TimestampDataType; // if 0, timestamps are stored as 32-bit integers; 
    // if 1, timestamps are stored as 64-bit integers;  supported by NeuroExplorer version 5.100 or greater

    int ContinuousDataType; // waveforms and continuous variables only, 
    // if 0, waveform and continuous values are stored as 16-bit integers; 
    // if 1, waveform and continuous values are stored as 32-bit floating point values in units specified in Units field

    double SamplingFrequency; // waveforms and continuous variables only, waveform or continuous variable sampling frequency in Hertz

    char Units[32]; // waveforms and continuous variables only, units that should be used for the variable values
    // not supported as of July 2017 (version 5.103)

    double ADtoUnitsCoefficient; // waveforms and continuous variables only, coefficient to convert from A/D values to units.
    // see formula below UnitsOffset below; ignored if ContinuousDataType == 1 

    double UnitsOffset; // waveforms and continuous variables only, 
    // this offset is used to convert A/D values in units:
    //  value_in_units = raw * ADtoUnitsCoefficient + UnitsOffset; ignored if ContinuousDataType == 1 

    unsigned long long NumberOfDataPoints; // waveform variable: number of data points in each wave
    // continuous variable: overall number of data points in the variable

    double PrethresholdTimeInSeconds; // waveform variables only, pre-threshold time in seconds
    // if waveform timestamp in seconds is t, 
    // then the timestamp of the first point of waveform is t - PrethresholdTimeInSeconds

    int MarkerDataType; // marker events only,
    // if 0, marker values are stored as strings; 
    // if 1, marker values are stored as 32-bit unsigned integers

    int	NumberOfMarkerFields; // marker events only, how many values are associated with each marker

    int	MarkerLength; // marker events only, how many characters are in each marker value; ignored if MarkerDataType is 1

    int ContinuousIndexOfFirstPointInFragmentDataType; //continuous variables only, 
    // if 0, indexes of first data point in fragments are stored as unsigned 32-bit integers; 
    // if 1, indexes of first data point in fragments are stored as unsigned 64-bit integers; not supported in NeuroExplorer as of July 2017 (version 5.103)

    char Padding[60]; // padding for future expansion (if any)
};

#pragma pack(pop)

